% 
% Lyapunov exponents for a New Keynesian Model with Markov Switching
%
% accompagning code for Neusser: Time-Varying Rational expectations Models
%
T=10^5;             % maximal number of iterations
tol = 10^-4;        % numerical accuracy
dim=2;              % dimension of the system
nstate=2;           % number of states
psim=0.6; qsim=0.4; % transition probabilities between states
%
% parameters of the model
%
beta=0.985; kappa=0.8; sig = 1; phi0=0;
% phisim=[0.01:0.05:4]; % values of phi
phisim=1.5              % value of phi
% phi0 and phi are the values of phi in the two states
%
% Representation and simulation of Markov switching process
%
t1=tic;
lyapsim = zeros(dim,length(phisim),length(psim));
Aswitch = zeros(dim,dim,nstate);
for i=1:length(phisim)
    phi = phisim(i); 
    for j=1:length(psim)
        for k=1:length(qsim)
            q=qsim(k); p=psim(j);
            P=[1-p p; q 1-q];  % state transition matrix
            pp = invariant(P); % invariant distribution
            B = zeros(2,2,3); 
            if p+q<=1
                % generates random numbers from a discrete probability distribution
                S=gendist([p q 1-p-q],T,1);
                % generates deterministic transition matrices
                B(:,:,1) = [0 1; 0 1]; B(:,:,2) = [1 0; 1 0]; B(:,:,3) = [1 0; 0 1];
            else
                % generates random numbers from a discrete probability distribution
                S=gendist([1-p 1-q p+q-1],T,1);
                % generates deterministic transition matrices
                B(:,:,1) = [1 0; 1 0]; B(:,:,2) = [0 1; 0 1]; B(:,:,3) = [0 1; 1 0];
            end
%
% System matrices in the two regimes
%
            Aswitch(:,:,1) = [beta+(kappa/sig) (phi0*beta-1)/sig ; -kappa 1]/beta;
            Aswitch(:,:,2) = [beta+(kappa/sig) (phi*beta-1)/sig;   -kappa 1]/beta;
% theoretical average Lyapunov exponents
            lyapave = (pp(1)*log(det(Aswitch(:,:,1))) + pp(2)*log(det(Aswitch(:,:,2))))/dim;
%
% initialization
%
            if rand<pp(1)
                sinit = [1 0];
            else
                sinit = [0 1];
            end
% initial state vector is sinit
            A = squeeze(Aswitch(:,:,sinit==1)); [qold,rold]=qr(A);
            qold=qold*diag(sign(diag(rold))); rold=q\A; % make positive diagonals in rold
            lambdaold=log(diag(rold)); % initial Lyapunov exponents
            sold=sinit;
            for t=2:T
                snew = sold*squeeze(B(:,:,S(t)));
                A = squeeze(Aswitch(:,:,snew==1))*qold;
                [qnew,rnew]=qr(A);
                qnew=qnew*diag(sign(diag(rnew))); rnew=qnew\A; % make positive diagonals in rnew
                lambdanew= (t-1)*lambdaold/t+log(diag(rnew))/t; % updated Lyapunov exponents
                if norm(lambdanew-lambdaold) < tol
                    break
                end
                sold = snew; qold=qnew; lambdaold=lambdanew;
            end
            lyapsim(:,i,j) = lambdanew;
            display([i,j])
        end
    end
end
clc
fprintf('\n')
fprintf('\n                                          Lyapunov exponents')
fprintf('\n                                          ------------------')
fprintf(' \n')
fprintf('\n method:                                          product QR')
fprintf(' \n')
fprintf(' dimension of system:                               %g\n',dim)
fprintf(' number of states:                                  %g\n',dim)
fprintf(' number of simulations:                             %g\n',t)
fprintf(' \n')
fprintf(' invariant distribution:                            %4.3f %4.3f\n',pp)
fprintf(' theoretical average Lyapunov exponent:             %4.3f\n',lyapave)
fprintf(' \n')
fprintf(' approximated average Lyapunov exponent:            %4.3f\n',mean(lambdanew))
fprintf(' approximated Lyapunov exponents:                   %4.3f %4.3f\n',lambdanew')
%
toc(t1);
